import time
from calendar import monthrange

from netforce.model import Model,fields,get_model
from netforce.access import get_active_company

class ReportLaborCost(Model):
    _name="clinic.report.labor.cost"
    _string="Report Labor Cost"
    _transient=True
    
    _fields={
        "date": fields.Date("Month"),
        "date_from": fields.Date("From", required=True),
        "date_to": fields.Date("To", required=True),
        "branch_id": fields.Many2One("clinic.branch","Branch"),
        "department_id": fields.Many2One("clinic.department","Department"),
        'report_type': fields.Selection([['cross','Cross'],['detail','Detail']],'Type'),
        'show_detail': fields.Boolean("Show Detail"),
    }

    def _get_date_from(self,context={}):
        year,month=time.strftime("%Y-%m").split("-")
        return '%s-%s-01'%(year,month)

    def _get_date_to(self,context={}):
        year,month,day=time.strftime("%Y-%m-%d").split("-")
        weekday, total_day=monthrange(int(year), int(month))
        return "%s-%s-%s"%(year,month,total_day)

    def default_get(self,field_names=None,context={},**kw):
        defaults=context.get("defaults",{})
        date_from=defaults.get("date_from", self._get_date_from())
        date_to=defaults.get("date_to", self._get_date_to())
        report_type=defaults.get("report_type","detail")
        print('defaults ', defaults)
        res={
            'date': time.strftime("%Y-%m-%d"),
            'date_from': date_from,
            'date_to': date_to,
            'show_detail': False,
            'report_type': report_type,
        }
        return res
    
    def get_report_data(self,ids,context={}):
        company_id=get_active_company()
        company=get_model("company").browse(company_id)
        defaults=self.default_get(context=context)
        date_from=defaults['date_from']
        date_to=defaults['date_to']
        branch_id=defaults.get("branch_id")
        department_id=defaults.get("department_id")
        show_detail=defaults['show_detail']
        report_type=defaults['report_type']
        dom=[]
        if ids:
            obj=self.browse(ids)[0]
            date_from=obj.date_from
            date_to=obj.date_to
            branch_id=obj.branch_id.id
            department_id=obj.department_id.id
            show_detail=obj.show_detail
            report_type=obj.report_type
        dom.append(['date','>=',date_from])
        dom.append(['date','<=',date_to])
        dom.append(['cycle_item_id.state','=', 'validated'])
        if branch_id:
            dom.append(['branch_id','=',branch_id])
        if department_id:
            dom.append(['department_id','=',department_id])
        date=date_from
        hdcases={}
        hdcase_obj=get_model("clinic.hd.case")
        dstates=dict(hdcase_obj._fields['state'].selection)
        print('dom ', dom)
        for hdcase in hdcase_obj.search_browse(dom):
            dpt=hdcase.department_id
            state=hdcase.state or ''
            if dpt.id not in hdcases.keys():
                hdcases[dpt.id]={
                    'name': dpt.name,
                    'department_id': dpt.id,
                    'branch_id': dpt.branch_id.id,
                    'qty':0,
                    'state':{},
                }
            #total for each department
            if state in ('waiting_payment','paid'):
                hdcases[dpt.id]['qty']+=1
            if not hdcases[dpt.id]['state'].get(state):
                hdcases[dpt.id]['state'][state]=0
            hdcases[dpt.id]['state'][state]+=1
        def replace_quote(dom=""):
            dom=dom.replace("False","false")
            dom=dom.replace("True","true")
            return dom.replace("'","\"")
        lines=[] 
        dom=[]
        dom.append(["date",">=",date_from])
        dom.append(["date","<=",date_to])
        for dpt_id, vals in hdcases.items():
            sub_lines=[]
            dom2=[['department_id','=',dpt_id]]
            qty=vals['qty'] or 0
            for state, sqty in vals['state'].items():
                ok=False
                dom3=[['state','=',state]] # last domain
                if state in ('waiting_payment','paid'):
                    ok=True
                dom4=dom+dom2+dom3
                state=dstates[state]
                sub_lines.append({
                    'name': state,
                    'qty': sqty,
                    'ok': ok,
                    'action_options': 'mode=list&search_domain=%s&tab_no=0'%replace_quote('%s'%(dom4)),
                })
            lines.append({
                'name': vals['name'],
                'branch_id': vals['branch_id'],
                'department_id': vals['department_id'],
                'qty': qty,
                'sub_lines': sub_lines,
            })
        sub_name=''
        if department_id:
            dpt=get_model("clinic.department").browse(department_id)
            sub_name="(%s)" % dpt.name or ""
        elif branch_id:
            branch=get_model("clinic.branch").browse(branch_id)
            sub_name="(%s)" % branch.name or ""
        total_hdcase=0
        for line in lines:
            qty=line['qty'] or 0
            total_hdcase+=qty
        dlines=[]
        nlines=[]
        if report_type=='cross':
            dom=[
                ['date','>=',date_from],
                ['date','<=',date_to],
                ['labor_cost_id.cycle_item_id.state','=','validated'],
            ]
            ddata={}
            ndata={}
            for line in get_model('clinic.labor.cost.line').search_browse(dom):
                cost_id=line.labor_cost_id
                citem=cost_id.cycle_item_id
                department=citem.department_id
                dpt_name=department.name or ''
                qty=line.qty or 0
                if line.type=='doctor':
                    if dpt_name not in ddata.keys():
                        ddata[dpt_name]=0
                    ddata[dpt_name]+=qty
                elif line.type=='nurse':
                    if dpt_name not in ndata.keys():
                        ndata[dpt_name]=set()
                    for hdcase in citem.hd_cases:
                        ndata[dpt_name].update({hdcase.id})
            dlines.append({
                'name': 'แพทย์',
                'qty': 'จำนวนครั้ง',
            })
            for dt_name,qty in ddata.items():
                dlines.append({
                    'name': dt_name,
                    'qty': qty,
                })
            nlines.append({
                'name': 'พยาบาล',
                'qty': 'จำนวนครั้ง',
            })
            for dt_name, hdcases in ndata.items():
                nlines.append({
                    'name': dt_name,
                    'qty': len(hdcases)
                })
        data={
            'company_name': '%s %s' % (company.name or "", sub_name),
            'date': date,
            'date_from': date_from,
            'date_to': date_to,
            'lines': lines,
            'dlines': dlines,
            'nlines': nlines,
            'total_hdcase': total_hdcase,
            'branch_id': branch_id,
            'department_id': department_id,
            'show_detail': show_detail,
            'report_type': report_type,
            #'ddata': ddata,
            #'ndata': ndata,
        }
        return data

    def onchange_date(self,context={}):
        data=context['data']
        date=data['date']
        year,month,day=date.split("-")
        weekday, total_day=monthrange(int(year), int(month))
        data['date_from']="%s-%s-01"%(year,month)
        data['date_to']="%s-%s-%s"%(year,month,total_day)
        return data

    def onchange_branch(self,context={}):
        data=context['data']
        data['department_id']=None
        return data

ReportLaborCost.register()
